# Using LRS Dockers

Here we describe how to interface with the Docker-based LRS modules from the local ROS installation of your Development System.  The intention is that this should be enough for a person knowledgeable about ROS to start developing their own functionality and interface with the available ROS modules.

See the following for an overview of the available functionality.  To access these pages, you need to log in to the portal.

- https://aiics.waraps.org/index.php/lrs-lrs_jupyter-md-djisdk-md/
- https://aiics.waraps.org/index.php/lrs-lrs_jupyter-md-single_agent-md/
- https://aiics.waraps.org/index.php/lrs-lrs_jupyter-md-single_agent_tst-md/


## Table of Contents

- [Scripts](#scripts)
- [How to start different parts](#how-to-start-different-parts)
- [Flying with Gamepad](#flying-with-gamepad)
- [Maintenance](#maintenance)


## Scripts

As long as you have followed the [installation instructions](../README.md) step by step, you now have the following scripts in your path.  These scripts are intended to be executed directly on the Development System but are used to *interact* with the Docker Container.  Concrete examples of using the scripts are given in the next section.

### wpull

Use `wpull` to pull (update to) the most recent version of the Development Environment docker image.  You should already have done this once, but you can do it again if you want to check for (and download) a new version.

The contents of this script:
```bash
#!/bin/bash

if [[ $(lsb_release -rs) == "18.04" ]]; then
    echo "Pulling gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-melodic:devel"
    docker pull gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-melodic:devel
fi

if [[ $(lsb_release -rs) == "20.04" ]]; then
    echo "Pulling gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-noetic:devel"
    docker pull gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-noetic:devel
fi
```

### wgrun

Start a docker container named `lrsdevel` and run the commands there. Use this if you have NVIDIA graphics.

The contents of this script:
```bash
#!/bin/bash

if [[ $(lsb_release -rs) == "18.04" ]]; then
docker run --gpus all \
  --network host \
  --security-opt apparmor:unconfined \
  --ulimit nofile=1024 \
  --ipc=host \
  -v /tmp/.X11-unix:/tmp/.X11-unix \
  -v /tmp:/exttmp \
  -e DISPLAY=unix${DISPLAY}\
  -e ROS_HOSTNAME=localhost \
  --rm -it --name lrsdevel gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-melodic:devel "$@"
fi

if [[ $(lsb_release -rs) == "20.04" ]]; then
docker run --gpus all \
  --network host \
  --security-opt apparmor:unconfined \
  --ulimit nofile=1024 \
  --ipc=host \
  -v /tmp/.X11-unix:/tmp/.X11-unix \
  -v /tmp:/exttmp \
  -e DISPLAY=unix${DISPLAY}\
  -e ROS_HOSTNAME=localhost \
  --rm -it --name lrsdevel gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-noetic:devel "$@"
fi
```


### wrun

Start a docker container named lrsdevel and run the command there. Use if you do not have NVIDIA graphics.

The contents of this script:
```bash
#!/bin/bash

if [[ $(lsb_release -rs) == "18.04" ]]; then
docker run \
  --network host \
  --ulimit nofile=1024 \
  --ipc=host \
  -v /tmp/.X11-unix:/tmp/.X11-unix \
  -v /tmp:/exttmp \
  -e DISPLAY=unix${DISPLAY}\
  -e ROS_HOSTNAME=localhost \
  --rm -it --name lrsdevel gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-melodic:devel "$@"
fi

if [[ $(lsb_release -rs) == "20.04" ]]; then
docker run \
  --network host \
  --ulimit nofile=1024 \
  --ipc=host \
  -v /tmp/.X11-unix:/tmp/.X11-unix \
  -v /tmp:/exttmp \
  -e DISPLAY=unix${DISPLAY}\
  -e ROS_HOSTNAME=localhost \
  --rm -it --name lrsdevel gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-noetic:devel "$@"
fi
```


### wexec

Execute commands in an already running `lrsdevel` docker container.

The contents of this script:
```bash
#!/bin/bash

docker exec -it lrsdevel "$@"
```


## How to start different parts

### Start a roscore and the docker container

There are two ways of running the roscore:

- Directly on the Development System.  This ensures that the roscore survives a restart of the Docker containers, which may be an advantage.  May or may not work if you run the Docker containers directly on a Windows or MacOS machine without an Ubuntu virtual machine.

  ```bash
  # Start the roscore here
  roscore
  # Start the Development Environment and make it run a simple bash shell 
  wgrun bash
  ```

- Inside the Docker container

  ```bash
  # Start the Development Environment and make it run the roscore
  wgrun roscore
  ```

### Loading config files for units

Assume we want to use unit /dji0 in Gränsö. Then do:
```bash
wexec roslaunch lrs_launch config.launch location:=granso no_airspace_booking:=true ns:=/dji0
```


### Starting Gazebo using Docker (optionally)

If you have an NVIDIA graphics card and support for that in docker, then you can if needed start gazebo (assuming that the container was already started with wgrun):

```bash
# Run roslaunch inside the docker container
wexec roslaunch lrs_wara_gazebo gazebo.launch
```

Load the dji model in gazebo:

```bash
# Run roslaunch inside the docker container
wexec roslaunch lrs_wara_gazebo dji_to_gazebo.launch name:=dji0 with_laser:=true x:=0
```


### Starting a simulator for dji0

Begin simulating a DJI Matrice, `dji0`:

```bash
wexec roslaunch lrs_launch djisimtst.launch dynamics:=true publish_pose:=true mpc:=true ns:=/dji0
```
In single-roscore mode the following holds:

- opunit.launch will start a program that converts from geopose to pose for all units.
- So the flag publish_pose should only be set to true if you run opunit.launch.

In multiple roscore mode, only geopose is sent to the op unit, so there we can choose freely if we want to generate the pose or not.


### Starting tstdisplayserver to show TSTs

Show TSTs graphically (when they are created/delegated):

```bash
wexec rosrun lrs_util_tst tstdisplayserver.py __ns:=/dji0
```

### Starting rviz

Use rviz to show the environment in Gränsö:
```bash
wexec roslaunch lrs_dji_sim rviz_granso.launch
```

Use rviz to show another location specified in the config. Currently only Gränsö has an orthophoto installed, though.
```bash
wexec roslaunch lrs_launch rviz_outside.launch
```

### Testing the dji0

On the Development System:

Watch the geopose:
```bash
rostopic echo /dji0/geopose
```

Try a Takeoff TST
```bash
wexec rosrun lrs_tst tstcommand.py --takeoff --exec __ns:=/dji0 --disp --op /dji0
```

Try a Flyto TST
```bash
wexec rosrun lrs_tst tstcommand.py --fly -x 100 -y 100 -z 12 --speed 3 --exec __ns:=/dji0 --disp --op /dji0
```

## Flying with Gamepad

To fly with a Gamepad, it is more convenient to run the roscore on the local machine.

These instructions assume that you haven't already started the Docker container.  If you have, stop it first.

On the development system:

```bash
roscore
```

And start the container as:
```bash
wgrun bash
```

On the local machine run the joy ros node:

Set the variable according to your particular Gamepad.  Example:
```bash
echo $JOY_EXTRA0 
usb-Logitech_Wireless_Gamepad_F710_2CAE9E79-joystick
```

To get the ID for the Gamepad do:
```bash
tompe@philo:~$ ls /dev/input/by-id/
usb-046d_C922_Pro_Stream_Webcam_B16031EF-event-if00
usb-CNFEH73M102010001C32_Integrated_Webcam_HD-event-if00
usb-ELAN_Touchscreen-event-if00
usb-HP_HP_USB_1000dpi_Laser_Mouse__LiteON_-event-mouse
usb-HP_HP_USB_1000dpi_Laser_Mouse__LiteON_-mouse
usb-Logitech_USB_Keyboard-event-if01
usb-Logitech_USB_Keyboard-event-kbd
usb-Logitech_Wireless_Gamepad_F710_2CAE9E79-event-joystick
usb-Logitech_Wireless_Gamepad_F710_2CAE9E79-joystick
```

Start the ROS driver for a joystick:
```bash
rosrun joy joy_node _dev:=/dev/input/by-id/${JOY_EXTRA0} _deadzone:=0.05 _autorepeat_rate:=50 _coalesce_interval:=0.1 __ns:=/dji0
```

Set up the simulation:
```bash
wexec roslaunch lrs_launch config.launch location:=granso no_airspace_booking:=true ns:=/dji0
wexec roslaunch lrs_launch djisimtst.launch dynamics:=true publish_pose:=true mpc:=true inair:=true ns:=/dji0
```

Start the joystick controller:
```bash
wexec rosrun lrs_dji joypose.py __ns:=/dji0
```


Start rviz:
```bash
wexec roslaunch lrs_launch rviz_outside.launch
```

Do take-off before flying. If take-off not done the altitude control does not work.

On the GamePad:
- Green Button: Manual Mode, fly with gamepad
- Red Button: Auto

## Maintenance

If you have ended up with a lot of running containers that interfere with each other you can stop all running containers with:

```bash
docker stop $(docker container ls -q)
```

And to clean up:
```bash
docker system prune
```

[Portainer](https://www.portainer.io/) is a useful tool if you want to manage your running containers.

##### Contact information

To ask questions about the development environment, contact `lrs-devenv@groups.liu.se`.